@@ -2,10 +2,11 @@
 
 
 from copy import copy as shallow_copy
+from functools import reduce
 
 
 debug = 0
-nmax = 15
+nmax = 16
 
 def set_nmax(n):
     """Sets *nmax* to the value *i*.
@@ -14,9 +15,26 @@ def set_nmax(n):
        should be between nmax/2 and nmax for efficient trees).
     """
     global nmax
+    if n % 4:
+        raise ValueError("nmax must be a multiple of 4.")    
     nmax = n
 
+    
 EMPTYSTYLE = {}
+style_pool = {():EMPTYSTYLE}
+
+def hash_style(style):
+    return tuple(sorted(style.items()))
+
+
+def as_style(d):
+    global style_pool
+    key = hash_style(d)
+    try:
+        return style_pool[key]
+    except KeyError:
+        style_pool[key] = d
+        return d
 
 
 # ---- Tree objects ----
@@ -44,6 +62,10 @@ class Single(Texel):
         clone.style = style
         return clone
 
+    def __setstate__(self, state):
+        self.__dict__ = state.copy()
+        self.style = as_style(self.style)
+
 
 class Text(Texel):
     is_text = 1
@@ -54,6 +76,16 @@ class Text(Texel):
 
     def __repr__(self):
         return "T(%s)" % repr(self.text)
+
+    def set_style(self, style):
+        clone = shallow_copy(self)
+        clone.style = style
+        return clone
+    
+    def __setstate__(self, state):
+        self.__dict__ = state.copy()
+        self.style = as_style(self.style)
+
 T = Text
 
 
@@ -73,7 +105,11 @@ class Group(_TexelWithChilds):
         sum)
 
     def __init__(self, childs):
-        self.childs = childs
+        # NOTE: childs are assumed to be lists and not to change!
+        # Since violations of these requirements can lead to difficult
+        # to find bugs, we are making a copy of the list here,
+        # although strictly this should not be necessary.
+        self.childs = list(childs[:])
         self.compute_weights()
 
     def __repr__(self):
@@ -91,7 +127,7 @@ class Container(_TexelWithChilds):
         
     def set_childs(self, childs):
         clone = shallow_copy(self)
-        clone.childs = childs
+        clone.childs = list(childs[:])
         clone.compute_weights()
         return clone
 
@@ -117,6 +153,10 @@ class NewLine(Single):
         clone.parstyle = style
         return clone
 
+    def __setstate__(self, state):
+        self.__dict__ = state.copy()
+        self.style = as_style(self.style)
+        self.parstyle = as_style(self.parstyle)
 
 
 class Tabulator(Single):
@@ -204,7 +244,11 @@ def iter_leaves(texel): # not needed, but might be useful in future
     """Iterate through all leaf-elements """
     l = [[texel]]
     i1 = 0
-    while l:
+    while 1:
+        while l and not l[-1]:
+            l.pop()
+        if not l:
+            break
         ll = l[-1]
         elem = ll[0]
         del ll[0]
@@ -213,8 +257,7 @@ def iter_leaves(texel): # not needed, but might be useful in future
         else:
             i2 = i1+length(elem)
             yield i1, i2, elem
-            while l and not l[-1]:
-                l.pop()
+            i1 = i2
 
 
 def groups(l):
@@ -241,18 +284,35 @@ def groups(l):
     if N == 0:
         return r
 
-    n = N // nmax
-    if n*nmax < N:
-        n = n+1
-    rest = n*nmax-N
+    assert nmax % 4 == 0 # nmax must be multiple of 4
+    s = 3*(nmax // 4)
+    
+    n = N // s
+    rest = N % s
+    assert N == rest+n*s
 
-    a = nmax-float(rest)/n
-    b = a
-    while l:
-        i = int(b+1e-10)
-        r.append(Group(l[:i]))
-        l = l[i:]
-        b = b-i+a
+    if rest > n*(nmax-s):
+        n += 1
+
+    elif s < 2*rest:
+        n += 1
+
+    m = N // n
+    t = N % n
+    assert N == n*m+t
+    
+    i1 = 0
+    for i in range(n-t):
+        i2 = i1+m
+        r.append(Group(l[i1:i2]))
+        i1 = i2
+
+    for i in range(t):
+        i2 = i1+m+1
+        r.append(Group(l[i1:i2]))
+        i1 = i2
+
+    assert i2 == len(l)
     return r
 
 
@@ -267,8 +327,8 @@ def _join(l1, l2):
        is_clean(__return__)
        calc_length(l1)+calc_length(l2) == calc_length(__return__)
     """
-    l1 = filter(length, l1) # strip off empty elements
-    l2 = filter(length, l2) #
+    l1 = list(filter(length, l1)) # strip off empty elements
+    l2 = list(filter(length, l2)) #
     if not l1:
         return l2
     if not l2:
@@ -314,8 +374,8 @@ def _fuse(l1, l2):
        is_homogeneous(l2)
        calc_length(l1)+calc_length(l2) == calc_length(__return__)
     """
-    l1 = filter(length, l1) # strip off empty elements
-    l2 = filter(length, l2) #
+    l1 = list(filter(length, l1)) # strip off empty elements
+    l2 = list(filter(length, l2)) #
     if not l1:
         return l2
     if not l2:
@@ -400,8 +460,8 @@ def insert(texel, i, stuff):
 def takeout(texel, i1, i2):
     """Takes out all content between *i1* and *i2*.
 
-    Returns the rest and the cut out piece (kernel), i.e.
-    G([a, b, c]).takeout(i1, i2) will return G([a, c]), b.
+    Returns the outer rest and the inner cut out piece, i.e.  G([a, b,
+    c]).takeout(1, 2) will return G([a, c]), b.
 
     *Texel* must be root efficient. Kernel and rest are guaranteed to
     be list efficient. Depths can change.
@@ -440,22 +500,24 @@ def takeout(texel, i1, i2):
     # point we only have G, C or T.
 
     if texel.is_group:
-        r1 = []; r2 = []; r3 = []; r4 = []
-        k1 = []; k2 = []; k3 = []
+        r1 = []; r2 = []; r3 = []; r4 = [] # outer rest
+        k1 = []; k2 = []; k3 = [] # inner kernel
         for j1, j2, child in iter_childs(texel):
+            # formal prove of if-conditions in notebook 26.08.2020
+            # collecting parts can still be simplified, see same entry
             if j2 <= i1:
                 r1.append(child)
-            elif i1 <= j1 <= j2 <= i2:
-                k2.append(child)
-            elif j1 <= i1 <= j2:
+            elif j1 < i1:
                 r, k = takeout(child, max(i1-j1, 0), min(i2-j1, length(child)))
                 r2.extend(r)
                 k1.extend(k)
-            elif j1 <= i2 <= j2:
+            elif j2 <= i2:
+                k2.append(child)
+            elif j1 < i2:
                 r, k = takeout(child, max(i1-j1, 0), min(i2-j1, length(child)))
                 r3.extend(r)
                 k3.extend(k)
-            elif i2 <= j1:
+            else:
                 r4.append(child)
         # Note that we are returning a list of elements which have
         # been in the content before. So even if texel is only root
@@ -463,10 +525,12 @@ def takeout(texel, i1, i2):
         # the list r1, r2, r3, r4 and k1, k2, k3 is
         # homogeneous. Therefore join gives us list efficient return
         # values.
-        if not is_clean(r2):
-            dump_list(r2)
-        if not is_clean(r3):
-            dump_list(r3)
+
+        if debug:
+            if not is_clean(r2):
+                dump_list(r2)
+            if not is_clean(r3):
+                dump_list(r3)
 
         tmp = fuse(r2, r3)
         return join(r1, tmp, r4), join(k1, k2, k3)
@@ -484,15 +548,51 @@ def takeout(texel, i1, i2):
         raise IndexError((i1, i2))
 
     elif texel.is_text:
-        r1 = texel.text[:i1]
-        r2 = texel.text[i2:]
-        r3 = texel.text[i1:i2]
-        s = texel.style
-        return [Text(r1+r2, s)], [Text(r3, s)]
+        r = texel.text[:i1]
+        s = texel.text[i1:i2]
+        t = texel.text[i2:]        
+        style = texel.style
+        return [Text(r+t, style)], [Text(s, style)]
 
     assert False
 
 
+    
+def checked_takeout(takeout):
+    def checked(texel, i1, i2):
+        assert is_root_efficient(texel)
+        __return__ = takeout(texel, i1, i2)
+        outer, inner = __return__
+
+        assert is_elementlist(__return__[0])
+        assert is_elementlist(__return__[1])
+        assert is_homogeneous(__return__[0])
+        assert is_homogeneous(__return__[1])
+        try:
+            assert calc_length(__return__[0])+i2-i1 == length(texel)
+        except:
+            
+            print(calc_length(__return__[0])+i2-i1, length(texel))
+            print("in:", texel, i1, i2)
+            print("outer:", outer)
+            print("inner:", inner)            
+            raise
+        assert calc_length(__return__[1]) == i2-i1
+        #out("takeout", texel, i1, i2)
+        #out(__return__[0])
+        #out(__return__[1])
+        #dump_list(__return__[0])
+        assert is_clean(__return__[0])
+        assert is_clean(__return__[1])
+        assert is_list_efficient(__return__[0])
+        assert is_list_efficient(__return__[1])
+        return __return__
+    return checked
+
+if debug:
+    takeout = checked_takeout(takeout)
+
+    
 def copy(root, i1, i2):
     """Copy all content of *root* between *i1* and *i2*.
 
@@ -751,7 +851,7 @@ def is_elementlist(l):
     """Returns True if *l* is a list of Elements.
     """
     if not type(l) in (tuple, list):
-        print "not a list or tuple", type(l), l.__class__ # XXX remove this
+        print("not a list or tuple", type(l), l.__class__) # XXX remove this
         return False
     return not False in [isinstance(x, Texel) for x in l]
 
@@ -775,18 +875,18 @@ def mindepth(l):
 
 
 def out(*args):
-    print repr(args)[1:-1]
+    print(repr(args)[1:-1])
     return True
 
 
 def dump(texel, i=0):
-    print (" "*i)+str(texel.__class__.__name__), texel.weights,
+    print((" "*i)+str(texel.__class__.__name__), texel.weights, end=' ')
     if texel.is_text:
-        print repr(texel.text), texel.style
+        print(repr(texel.text), texel.style)
     elif isinstance(texel, NewLine):
-        print texel.parstyle
+        print(texel.parstyle)
     else:
-        print
+        print()
     if texel.is_group or texel.is_container:
         for i1, i2, child in iter_childs(texel):
             dump(child, i+2)
@@ -794,19 +894,19 @@ def dump(texel, i=0):
 
 
 def dump_list(l):
-    print "Dumping list (efficient: %s)" % is_list_efficient(l)
+    print("Dumping list (efficient: %s)" % is_list_efficient(l))
     for i, element in enumerate(l):
-        print "Dumping element no. %i" % i,
-        print "(efficient: %s)" % is_efficient(element)
+        print ("Dumping element no. %i" % i, end=' ')
+        print ("(efficient: %s)" % is_efficient(element))
         dump(element)
     return True
 
 
 # ---- Testing ----
 
-if debug: # enable contract checking
-     import contract
-     contract.checkmod(__name__)
+if 0 and debug: # enable contract checking
+    import contract
+    contract.checkmod(__name__)
 
 
 def test_00():
@@ -826,7 +926,7 @@ def test_00():
 
 def test_01():
     "growth in insert"
-    set_nmax(3)
+    set_nmax(4)
     g = Group([T("a"), T("b"), T("c")])
     assert depth(g) == 1
     assert get_pieces(g) == ['a', 'b', 'c']
@@ -836,7 +936,7 @@ def test_01():
 
 def test_02():
     "maintaining tree efficency in insert / takeout"
-    set_nmax(3)
+    set_nmax(4)
     texel = Group([])
     import random
     for i in range(100):
@@ -944,4 +1044,65 @@ def test_07():
     assert compute_hull(fraction, n-1, n) == (0, n)
 
 
+def test_08():
+    "takeout"
+    s1 = as_style(dict(color='red'))
+    s2 = as_style(dict(color='blue'))
+    t1 = T("0123456789", style=s1)
+    t2 = T("ABC", style=s2)
+    g = G((t1, t2))
+    r, k = takeout(g, 5, 12)    
+    assert get_text(grouped(r)) == "01234C"
+    assert get_text(grouped(k)) == "56789AB"
+
+    t1 = T("0123456789")
+    t2 = T("ABC")
+    g = G((t1, t2))    
+    r, k = takeout(g, 5, 12)
+    assert get_pieces(grouped(r)) == ['01234C']
+    assert get_text(grouped(k)) == "56789AB"
+
     
+def test_09():
+    "pickle"
+    s1 = as_style(dict(color='red'))
+    s2 = as_style(dict(color='blue'))
+    t1 = T("012345678", style=s1)
+    t2 = T("ABC", style=s2)
+    g = G((t1, t2))
+    from pickle import dumps, loads 
+    t1_ = loads(dumps(t1))
+    assert t1.style is t1_.style
+    assert t1.text == t1_.text    
+    g_ = loads(dumps(g))
+    t1_, t2_ = g_.childs
+    assert not t1_ is t1
+    assert not t2_ is t2
+    assert t1.style is t1_.style
+    assert t2.style is t2_.style
+    assert t1.text == t1_.text    
+    assert t2.text == t2_.text    
+
+
+def test_10():
+    "set_style"
+    s1 = as_style(dict(color='red'))
+    s2 = as_style(dict(color='blue'))
+    t1 = T("012345678", style=s1)
+    t2 = t1.set_style(s2)
+    assert t1.style == dict(color='red')
+    assert t2.style == dict(color='blue')
+
+
+def test_11():
+    "set_parstyle"
+    s1 = as_style(dict(base='h1'))
+    s2 = as_style(dict(base='h2'))
+    nl1 = NL.set_parstyle(s1)
+    nl2 = NL.set_parstyle(s2)
+    print(NL.parstyle)
+    assert NL.parstyle == dict()
+    assert nl1.parstyle == dict(base='h1')
+    assert nl2.parstyle == dict(base='h2')
+
+
