@@ -1,10 +1,13 @@
 # -*- coding: latin-1 -*-
 
 
+from __future__ import print_function
+from __future__ import absolute_import
 from ..textmodel.viewbase import ViewBase, overridable_property
 from ..textmodel.modelbase import Model
 from ..textmodel.textmodel import dump_range
 from ..textmodel import TextModel
+from six.moves import range
 
 
 debug = 0
@@ -14,7 +17,7 @@ def undo(info):
     if callable(info[0]):
         func = info[0]
         args = info[1:]    
-        redo = apply(func, args)
+        redo = func(*args)
     else:
         redo = []
         for child in reversed(info):
@@ -56,9 +59,9 @@ class TextView(ViewBase, Model):
     def join_undo(self, info2, info1):
         # we are joining similar undo entries
         if 0:
-            print "join"
-            print "info1=", info1
-            print "info2=", info2
+            print("join")
+            print("info1=", info1)
+            print("info2=", info2)
         if info1[0] == info2[0]:
             if info1[0] == self._remove:
                 i1, i2 = info1[1:]
@@ -155,6 +158,20 @@ class TextView(ViewBase, Model):
         styles = self.model.set_styles(i, styles)
         return self._set_styles, i, styles
 
+    def set_parstyle(self, i, style):
+        styles = self.model.set_parstyle(i, style)
+        info = self._set_parstyles, i, styles
+        self.add_undo(info)
+
+    def set_parproperties(self, i1, i2, **properties):
+        styles = self.model.set_parproperties(i1, i2, **properties)
+        info = self._set_parstyles, i1, styles
+        self.add_undo(info)
+
+    def _set_parstyles(self, i, styles):
+        styles = self.model.set_parstyles(i, styles)
+        return self._set_parstyles, i, styles
+    
     def transform(self, fun):
         # Apply a tranforming function to the texeltree. Can change
         # index.
@@ -170,6 +187,9 @@ class TextView(ViewBase, Model):
         self.rebuild()
         return self._set_texel, old
 
+    def get_maxw(self):
+        return self._maxw
+    
     def set_maxw(self, maxw):
         if maxw == self._maxw:
             return
@@ -179,6 +199,71 @@ class TextView(ViewBase, Model):
         self.Refresh()
         self.notify_views('maxw_changed')
 
+    def indent_rows(self, firstrow, lastrow, n=4):
+        model = self.model
+        has_selection = self.has_selection()
+        if has_selection:
+            s1, s2 = self.selection
+        else:
+            s1 = s2 = 0
+        index = self.index
+        for line in range(lastrow, firstrow-1, -1): # indent
+            i = model.linestart(line)
+            model.insert_text(i, ' '*n)
+            if index >= i:
+                index += n
+            if s1 >= i:
+                s1 += n
+            if s2 >= i:
+                s2 += n
+        self.index = index
+        if has_selection:
+            self.selection = (s1, s2)            
+        info = self._dedent_rows, firstrow, lastrow, n
+        self.add_undo(info)
+
+    def _dedent_rows(self, firstrow, lastrow, n):
+        has_selection = self.has_selection()
+        if has_selection:
+            s1, s2 = self.selection
+        else:
+            s1 = s2 = 0
+        index = self.index
+        memo = []
+        model = self.model
+        for line in reversed(list(range(firstrow, lastrow+1))):
+            i = model.linestart(line)
+            for j in range(i, i+n+1):
+                if j > len(model): break
+                if model.get_text(j, j+1) != ' ': break
+            memo.append(model.remove(i, j))
+            if index > i:
+                index = i+max(0, index-j)
+            if s1 > i:
+                s1 = i+max(0, s1-j)
+            if s2 > i:
+                s2 = i+max(0, s2-j)
+        self.index = index
+        if has_selection:
+            self.selection = (s1, s2)
+            
+        assert len(memo) == lastrow-firstrow+1
+        return self._undo_dedent, firstrow, memo, n
+        
+    def dedent_rows(self, firstrow, lastrow, n=4):
+        info = self._dedent_rows(firstrow, lastrow, n)
+        self.add_undo(info)
+
+    def _undo_dedent(self, firstrow, memo, n):
+        model = self.model
+        lastrow = firstrow+len(memo)-1
+        memo = list(memo)
+        for line in reversed(list(range(firstrow, lastrow+1))):
+            i = model.linestart(line)
+            model.insert(i, memo[0])
+            memo = memo[1:]
+        return self._dedent_rows, firstrow, lastrow, n
+
     def compute_index(self, x, y):
         if y >= self.layout.height:
             return len(self.model)-1
@@ -198,29 +283,38 @@ class TextView(ViewBase, Model):
         index = self.index
         layout = self.layout
         style = self.current_style()
+        parstyle = model.get_parstyle(index)
         row, col = self.current_position()
         rect = layout.get_rect(index, 0, 0)
         x = rect.x1
         y = rect.y1
+        if self.has_selection():
+            s1, s2 = sorted(self.selection)
+            e1, e2 = layout.extend_range(s1, s2)
+        else:
+            s1 = s2 = e1 = e2 = index
 
         def del_selection():
             if self.has_selection():
-                s1, s2 = sorted(self.selection)
-                s1, s2 = layout.extend_range(s1, s2)
-                self.remove(s1, s2)
+                self.remove(e1, e2)
 
         if action == 'dump_info':
-            s1, s2 = sorted(self.selection)
-            s1, s2 = layout.extend_range(s1, s2)
-            dump_range(model.texel, s1, s2)
+            dump_range(model.texel, e1, e2)
             row, col = model.index2position(index)
-            print "index=", index
-            print "row=", row
-            print "col=", col
+            print("index=", index)
+            print("row=", row)
+            print("col=", col)
 
         elif action == 'dump_boxes':
             layout.dump_boxes(0, 0, 0)
-            
+        elif action == 'indent':
+            row1 = model.index2position(s1)[0]
+            row2 = model.index2position(s2)[0]
+            self.indent_rows(row1, row2)
+        elif action == 'dedent':
+            row1 = model.index2position(s1)[0]
+            row2 = model.index2position(s2)[0]
+            self.dedent_rows(row1, row2)            
         elif action == 'move_word_end':
             i = index
             n = len(model)
@@ -286,18 +380,28 @@ class TextView(ViewBase, Model):
         elif action == 'select_all':
             self.selection = (0, len(model))
         elif action == 'insert_newline':
-            self.insert(index, self._TextModel('\n', **style))
+           tmp = self._TextModel('\n', **style)
+           tmp.set_parstyle(0, parstyle)
+           self.insert(index, tmp)            
         elif action == 'insert_newline_indented':
             i = model.linestart(row)
             s = model.get_text(i, index)
             l = s[:len(s)-len(s.lstrip())]
-            self.insert(index, self._TextModel('\n'+l, **style))
+            tmp = self._TextModel('\n'+l, **style)
+            tmp.set_parstyle(0, parstyle)
+            self.insert(index, tmp)            
         elif action == 'backspace':
-            del_selection()
-            i = self.index
-            if i>0:
-                j1, j2 = layout.extend_range(i-1, i)
-                self.remove(j1, j2)
+            if self.has_selection():
+                j1, j2 = layout.extend_range(s1-1, s2)
+                if j2 != e2:
+                    self.remove(e1, e2)
+                else:
+                    self.remove(j1, j2)
+            else:
+                i = self.index
+                if i>0:
+                    j1, j2 = layout.extend_range(i-1, i)
+                    self.remove(j1, j2)
         elif action == 'copy':
             self.copy()
         elif action == 'paste':
@@ -318,6 +422,9 @@ class TextView(ViewBase, Model):
             self.redo()
         elif action == 'del_line_end':
             i = model.linestart(row)+model.linelength(row)-1
+            if i == index:
+                i += 1
+            self.to_clipboard(model[index:i])
             self.remove(index, i)
         elif action == 'del_word_left':
             # find the beginning of the word
@@ -338,12 +445,32 @@ class TextView(ViewBase, Model):
         self.Refresh()
 
     def copy(self):
-        raise NotImplemented()
+        if not self.has_selection():
+            return        
+        s1, s2 = self.get_selected()[0] # XXX Assuming just one region
+        part = self.model[s1:s2]
+        self.to_clipboard(part)
 
     def paste(self):
-        raise NotImplemented()
+        if self.has_selection():
+            for s1, s2 in self.get_selected():
+                self.model.remove(s1, s2)
+                self.index = s1
+        textmodel = self.read_clipboard()
+        if textmodel is not None:
+            self.insert(self.index, textmodel)
+                
 
     def cut(self):
+        if self.has_selection():
+            self.copy()
+            for s1, s2 in self.get_selected():
+                self.remove(s1, s2)
+         
+    def to_clipboard(self, textmodel):
+        raise NotImplemented()
+
+    def read_clipboard(self):
         raise NotImplemented()
          
     def select_word(self, x, y):
